@@ -1,3 +1,20 @@
+# Copyright (C) 2013-2015 Martin Drees
+#
+# This file is part of darch.
+#
+# darch is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# darch is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with darch. If not, see <http://www.gnu.org/licenses/>.
+
 #' Sets the list of RBMs
 #'
 #' @param darch A instance of the class \code{\link{DArch}}.
@@ -6,14 +23,13 @@
 #' @seealso \code{\link{DArch}}
 #' 
 #' @export
-#' @docType methods
-#' @rdname setRBMList-methods
 #' @include darch.R
+#' @rdname setRBMList-methods
 setGeneric("setRBMList<-",function(darch,value){standardGeneric("setRBMList<-")})
 
 #' @rdname setRBMList-methods
 #' @aliases setRBMList<-,DArch-method
-#' @name setRBMList
+#' @name setRBMList<-
 setReplaceMethod(
   f="setRBMList",
   signature="DArch",
@@ -70,8 +86,8 @@ setReplaceMethod(
   signature="DArch",
   definition=function(darch,index,value){
     layer = list()
-    if(darch@ff){
-      if(!is.ff(darch@layers[[index]])){
+    if (darch@ff){
+      if (!is.ff(darch@layers[[index]])){
         layer[[1]] <- ff(vmode="double",dim=dim(value))
       }
       layer[[1]][] <- value
@@ -105,8 +121,16 @@ setReplaceMethod(
   f="setLayerWeights",
   signature="DArch",
   definition=function(darch,index,value){
-    if(darch@ff){
-      if(!is.ff(darch@layers[[index]])){
+    # normalize weights
+    if (darch@normalizeWeights)
+    {
+      value[1:(nrow(value) - 1),] <-
+        apply(value[1:(nrow(value) - 1),, drop = F], 2,
+              function (e) { e/norm(e, "2") })
+    }
+    
+    if (darch@ff){
+      if (!is.ff(darch@layers[[index]])){
         darch@layers[[index]][[1]] <- ff(vmode="double",dim=dim(value))
       }
       darch@layers[[index]][[1]][] <- value
@@ -169,9 +193,9 @@ setReplaceMethod(
   f="setLayerField",
   signature="DArch",
   definition=function(darch,layerIndex,fieldIndex,value){
-    if(class(value) == "matrix"){
-      if(darch@ff){
-        if(length(darch@layers[[layerIndex]]) < fieldIndex || !is.ff(darch@layers[[layerIndex]][[fieldIndex]])){
+    if (class(value) == "matrix"){
+      if (darch@ff){
+        if (length(darch@layers[[layerIndex]]) < fieldIndex || !is.ff(darch@layers[[layerIndex]][[fieldIndex]])){
           darch@layers[[layerIndex]][[fieldIndex]] <- ff(vmode="double",dim=dim(value))
         }
         darch@layers[[layerIndex]][[fieldIndex]][] <- value
@@ -312,4 +336,140 @@ setReplaceMethod(
     darch@cancelMessage <-value
     return (darch)
   }
+)
+
+#' Sets the dropout rate for the input layer.
+#'
+#' @param darch A instance of the class \code{\link{DArch}}.
+#' @param value Dropout rate for the input layer
+#' @usage setDropoutInputLayer(darch) <- value
+#' @return The darch with the updated dropout rate
+#' @seealso \code{\link{DArch}}
+#' 
+#' @export
+#' @docType methods
+#' @rdname setDropoutInputLayer-methods
+#' @include darch.R
+setGeneric("setDropoutInputLayer<-",function(darch,value){standardGeneric("setDropoutInputLayer<-")})
+
+#' @rdname setDropoutInputLayer-methods
+#' @aliases setDropoutInputLayer<-,DArch-method
+#' @name setDropoutInputLayer
+setReplaceMethod(
+  f="setDropoutInputLayer",
+  signature="DArch",
+  definition=function(darch,value){
+    darch@dropoutInput <- value
+    return (darch)
+  }
+)
+
+#' Sets the dropout rate for the hidden layers.
+#'
+#' @param darch A instance of the class \code{\link{DArch}}.
+#' @param value Dropout rate for the hidden layers
+#' @usage setDropoutHiddenLayers(darch) <- value
+#' @return The darch with the updated dropout rate
+#' @seealso \code{\link{DArch}}
+#' 
+#' @export
+#' @docType methods
+#' @rdname setDropoutHiddenLayers-methods
+#' @include darch.R
+setGeneric("setDropoutHiddenLayers<-",function(darch,value){standardGeneric("setDropoutHiddenLayers<-")})
+
+#' @rdname setDropoutHiddenLayers-methods
+#' @aliases setDropoutHiddenLayers<-,DArch-method
+#' @name setDropoutHiddenLayers
+setReplaceMethod(
+  f="setDropoutHiddenLayers",
+  signature="DArch",
+  definition=function(darch,value){
+    darch@dropoutHidden <- value
+    return (darch)
+  }
+)
+
+#' Set dropout mask usage
+#' 
+#' Set whether the same dropout mask should be used for all batches of an epoch,
+#' or a new set of masks should be generated for each batch (default).
+#' 
+#' @param darch A instance of the class \code{\linkS4class{DArch}}.
+#' @param value Boolean value indicating whether to use a dropout mask for each
+#'   epoch.
+#'   
+#' @export
+#' @include darch.R
+setGeneric("setDropoutOneMaskPerEpoch<-",function(darch,value){standardGeneric("setDropoutOneMaskPerEpoch<-")})
+
+#' Set dropout mask usage
+#' 
+#' @inheritParams setDropoutOneMaskPerEpoch<-
+#' @seealso \link{setDropoutOneMaskPerEpoch<-}
+#' @export
+setReplaceMethod(
+  f="setDropoutOneMaskPerEpoch",
+  signature="DArch",
+  definition=function(darch,value){
+    darch@dropoutOneMaskPerEpoch <- value
+    return (darch)
+  }
+)
+
+#' Set the dropout masks.
+#'
+#' @param darch A instance of the class \code{\link{DArch}}.
+#' @param value List of dropout masks
+#' @usage setDropoutMasks(darch) <- value
+#' @return The darch with the updated dropout masks
+#' @seealso \code{\link{DArch}}
+#' 
+#' @export
+#' @docType methods
+#' @rdname setDropoutMasks-methods
+#' @include darch.R
+setGeneric("setDropoutMasks<-",function(darch,value){standardGeneric("setDropoutMasks<-")})
+
+#' @rdname setDropoutMasks-methods
+#' @aliases setDropoutMasks<-,DArch-method
+#' @name setDropoutMasks
+setReplaceMethod(
+  f="setDropoutMasks",
+  signature="DArch",
+  definition=function(darch,value){
+    darch@dropoutMasks <- value
+    return (darch)
+  }
+)
+
+#' Set the dropout mask for the given layer.
+#' 
+#' Sets the dropout mask to be applied to the weights between layer i and i+1,
+#' for 0 < i < numLayers. For i = 0, sets the dropout mask for the input layer,
+#' which will be applied to the initiali input data.
+#'
+#' @param darch A instance of the class \code{\link{DArch}}.
+#' @param i Layer index or 0 for input layer.
+#' @param value Dropout mask for the given layer.
+#' @usage setDropoutMask(darch, i) <- value
+#' @return The darch with the updated dropout mask
+#' @seealso \code{\link{DArch}}
+#' 
+#' @export
+#' @docType methods
+#' @rdname setDropoutMask-methods
+#' @include darch.R
+setGeneric("setDropoutMask<-",function(darch,i,value){standardGeneric("setDropoutMask<-")})
+
+#' @rdname setDropoutMask-methods
+#' @aliases setDropoutMask<-,DArch-method
+#' @name setDropoutMask
+setReplaceMethod(
+  f="setDropoutMask",
+  signature="DArch",
+  definition=function(darch,i,value){
+    darch@dropoutMasks[[i+1]] <- value
+    return (darch)
+  }
 )
\ No newline at end of file
