@@ -1,4 +1,21 @@
-#' Generates the rbm's for the pre-training.
+# Copyright (C) 2013-2015 Martin Drees
+#
+# This file is part of darch.
+#
+# darch is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# darch is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with darch. If not, see <http://www.gnu.org/licenses/>.
+
+#' Generates the RBMs for the pre-training.
 #' 
 #' Used the layer sizes from the DArch object to create the RBM objects for the
 #' pre-training. 
@@ -6,7 +23,7 @@
 #' @param darch A instance of the class \code{\link{DArch}}.
 #' @param layers An array with the sizes of the layers
 #' @param genWeightFunc The function for generating the weight matrices
-#' @return The DArch object with the generated rbm's
+#' @return The DArch object with the generated RBMs
 #' 
 #' @seealso \code{\link{DArch}}
 #'          \code{\link{RBM}}
@@ -35,7 +52,7 @@ setMethod(
       hidden <- layers[[(i+1)]]
       rbm <- newRBM(visible,hidden,getBatchSize(darch),getFF(darch),flog.logger()$threshold,genWeightFunc)
       darch@rbmList[i] <- rbm
-      darch <- addLayer(darch,getWeights(rbm),getHiddenBiases(rbm),sigmoidUnit)				
+      darch <- addLayer(darch,getWeights(rbm),getHiddenBiases(rbm),sigmoidUnitDerivative)				
     }
     return(darch)
   }
