@@ -1,3 +1,20 @@
+# Copyright (C) 2013-2015 Martin Drees
+#
+# This file is part of darch.
+#
+# darch is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# darch is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with darch. If not, see <http://www.gnu.org/licenses/>.
+
 #' Conjugate gradient for a autoencoder network
 #' 
 #' This function trains a \code{\link{DArch}} autoencoder network with the 
@@ -17,20 +34,17 @@
 #' @param darch A instance of the class \code{\link{DArch}}.
 #' @param trainData The training data matrix
 #' @param targetData The labels for the training data
-#' @param epoch The actual epoch of the training
 #' @param length Numbers of line search 
 #' 
 #' @return The trained \code{\link{DArch}} object.
-#' @usage minimizeAutoencoder(darch,trainData,targetData,epoch,length)
 #' @seealso \code{\link{DArch}}
 #' \code{\link{fineTuneDArch}}
 #' 
-#' @docType methods
-#' @rdname minimizeAutoencoder
 #' @include darch.R
 #' @export
-minimizeAutoencoder <- function(darch,trainData,targetData,epoch,length){
-    
+minimizeAutoencoder <- function(darch,trainData,targetData,length){
+  
+  matMult <- get("matMult", darch.env)
   # Function for gradients ###############################
   fr <- function(par,darch,dims,data){
     
@@ -50,8 +64,8 @@ minimizeAutoencoder <- function(darch,trainData,targetData,epoch,length){
       endPos <- endPos + dims[[i]][1]*dims[[i]][2]
       weights[[i]] <- matrix(par[startPos:endPos],dims[[i]][1],dims[[i]][2])
       startPos <- endPos+1
-      layer <- getLayer(darch,i)
-      ret <- layer[[2]](d,weights[[i]]) # noch eine funktion getLayerFunction() einfügen
+      ret <- getLayerFunction(darch, i)(d,weights[[i]])
+      getLayerFunction()
       outputs[[i]] <- ret[[1]]
       d <- ret[[1]]
       derivatives[[i]] <- ret[[2]]
@@ -63,18 +77,18 @@ minimizeAutoencoder <- function(darch,trainData,targetData,epoch,length){
     
     ix <- 1/nrow(data)*(output-data)
     out <- cbind(outputs[[i-1]],rep(1,nrow(outputs[[i-1]])))
-    gradients[[length]] <- t(out)%*%ix
+    gradients[[length]] <- matMult(t(out), ix)
     
     for(i in (length-1):1){
       derivatives[[i]] <- cbind(derivatives[[i]],rep(1,nrow(derivatives[[i]])))
-      ix <- (ix%*%t(weights[[i+1]]))* derivatives[[i]] # outputs[[i]]*(1-outputs[[i]])
+      ix <- (matMult(ix, t(weights[[i+1]])))* derivatives[[i]] # outputs[[i]]*(1-outputs[[i]])
       ix <- ix[,1:(dim(ix)[2]-1)]
-      if(i > 1){
+      if (i > 1){
         out <- cbind(outputs[[i-1]],rep(1,nrow(outputs[[i-1]])))
-        gradients[[i]] <- t(out)%*% ix
+        gradients[[i]] <- matMult(t(out), ix)
       }else{
         d <- cbind(data,rep(1,numRows))
-        gradients[[i]] <- t(d)%*% ix
+        gradients[[i]] <- matMult(t(d), ix)
       }
     }
     
@@ -96,7 +110,7 @@ minimizeAutoencoder <- function(darch,trainData,targetData,epoch,length){
   for(i in 1:numLayers){
     weights <- getLayerWeights(darch,i)
     dims[[i]] <- dim(weights)
-    par <- c(par,c(weights))		
+    par <- c(par,c(weights))
   }
   
   # optimize
