@@ -1,3 +1,20 @@
+# Copyright (C) 2013-2015 Martin Drees
+#
+# This file is part of darch.
+#
+# darch is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# darch is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with darch. If not, see <http://www.gnu.org/licenses/>.
+
 #' Conjugate gradient for a classification network
 #' 
 #' This function trains a \code{\link{DArch}} classifier network with the conjugate
@@ -22,13 +39,11 @@
 #' @param darch A instance of the class \code{\link{DArch}}.
 #' @param trainData The training data matrix
 #' @param targetData The labels for the training data
-#' @param epoch The actual epoch of the training
 #' @param length Numbers of line search 
 #' @param switchLayers Indicates when to train the full network instead of only 
 #' the upper two layers
 #' 
 #' @return The trained \code{\link{DArch}} object.
-#' @usage minimizeClassifier(darch,trainData,targetData,epoch,length,switchLayers)
 #' 
 #' @seealso \code{\link{DArch}}
 #' 
@@ -36,7 +51,8 @@
 #' @rdname minimizeClassifier
 #' @include darch.R
 #' @export
-minimizeClassifier <- function(darch,trainData,targetData,epoch,length,switchLayers){
+minimizeClassifier <- function(darch,trainData,targetData,length,switchLayers){
+  matMult <- get("matMult", darch.env)
   
   # Function for gradients ###############################
   fr <- function(par,darch,dims,data,target,epochSwitch){
@@ -49,7 +65,7 @@ minimizeClassifier <- function(darch,trainData,targetData,epoch,length,switchLay
     derivatives <- list()
     weights <- list()
     
-    if(epochSwitch){
+    if (epochSwitch){
       d <- data	
       # Calculating the outputs
       for(i in 1:length){
@@ -57,8 +73,7 @@ minimizeClassifier <- function(darch,trainData,targetData,epoch,length,switchLay
         endPos <- endPos + dims[[i]][1]*dims[[i]][2]
         weights[[i]] <- matrix(par[startPos:endPos],dims[[i]][1],dims[[i]][2])
         startPos <- endPos+1
-        layer <- getLayer(darch,i)
-        ret <- layer[[2]](d,weights[[i]]) # noch eine funktion getLayerFunction() einfügen
+        ret <- getLayerFunction(darch, i)(d,weights[[i]])
         outputs[[i]] <- ret[[1]]
         d <- ret[[1]]
         derivatives[[i]] <- ret[[2]]
@@ -69,32 +84,31 @@ minimizeClassifier <- function(darch,trainData,targetData,epoch,length,switchLay
       
       ix <- output-target
       out <- cbind(outputs[[i-1]],rep(1,nrow(outputs[[i-1]])))
-      gradients[[length]] <- t(out)%*%ix
+      gradients[[length]] <- matMult(t(out), ix)
       
       for(i in (length-1):1){
         derivatives[[i]] <- cbind(derivatives[[i]],rep(1,nrow(derivatives[[i]])))
-        ix <- (ix%*%t(weights[[i+1]]))* derivatives[[i]] # outputs[[i]]*(1-outputs[[i]])
+        ix <- (matMult(ix, t(weights[[i+1]])))* derivatives[[i]] # outputs[[i]]*(1-outputs[[i]])
         ix <- ix[,1:(dim(ix)[2]-1)]
-        if(i > 1){
+        if (i > 1){
           out <- cbind(outputs[[i-1]],rep(1,nrow(outputs[[i-1]])))
-          gradients[[i]] <- t(out)%*% ix
+          gradients[[i]] <- matMult(t(out), ix)
         }else{
           d <- cbind(data,rep(1,numRows))
-          gradients[[i]] <- t(d)%*% ix
+          gradients[[i]] <- matMult(t(d), ix)
         }
       }
     }else{
       d <- cbind(data,rep(1,numRows))
       endPos <- endPos + dims[[1]][1]*dims[[1]][2]
       weights <- matrix(par[startPos:endPos],dims[[1]][1],dims[[1]][2])
-      layer <- getLayer(darch,length(getLayers(darch)))
-      ret <- layer[[2]](d,weights) # noch eine funktion getLayerFunction() einfügen
+      ret <- getLayerFunction(darch, length(getLayers(darch)))(d,weights)
       output <- ret[[1]]
       
       f = -sum(target*log(output))
       
       ix <- output-target
-      gradients[[1]] <- t(d)%*%ix
+      gradients[[1]] <- matMult(t(d), ix)
     }
     
     ret <- c(f)
@@ -113,10 +127,7 @@ minimizeClassifier <- function(darch,trainData,targetData,epoch,length,switchLay
   par <- c()
   dims <- list()
   
-  epochSwitch <- TRUE
-  if(epoch < switchLayers){
-    epochSwitch <- FALSE
-  }
+  epochSwitch <- getEpochs(darch) >= switchLayers
   
   if(epochSwitch){
     
@@ -134,7 +145,7 @@ minimizeClassifier <- function(darch,trainData,targetData,epoch,length,switchLay
     
     for(i in 1:(length-1)){
       trainData <- cbind(trainData,rep(1,numRows))
-      trainData <- 1/(1 + exp(-trainData %*% layers[[i]][[1]][]))
+      trainData <- 1/(1 + exp(matMult(-trainData, layers[[i]][[1]][])))
     }
     
     weights <- getLayerWeights(darch,length)
