@@ -1,3 +1,20 @@
+# Copyright (C) 2013-2015 Martin Drees
+#
+# This file is part of darch.
+#
+# darch is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# darch is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with darch. If not, see <http://www.gnu.org/licenses/>.
+
 #' Constructor function for \code{\link{DArch}} objects.
 #' 
 #' Generate a new \code{\link{DArch}} object with the given parameters.
@@ -6,7 +23,7 @@
 #' It is recommended to use this function for generating a new 
 #' \code{\link{DArch}} object, because this function generates and sets all the
 #' necessary parameters like the internally used \code{\link{RBM}} networks, 
-#' the list of statistiks (\code{stats}) etc.
+#' the list of statistics (\code{stats}) etc.
 #' 
 #' @param layers Array of layer sizes.
 #' @param batchSize Size of the batches
@@ -32,7 +49,10 @@ newDArch <- function(layers,batchSize,ff=FALSE,
   setFF(darch) <- ff
   setBatchSize(darch) <- batchSize  
   setGenWeightFunction(darch) <- genWeightFunc
-  setStats <- list()
+  setStats(darch) <-
+    list("dataErrors" = list("raw"=c(), "class" = c()),
+         "validErrors" = list("raw"=c(), "class" = c()),
+         "times" = c(), "preTrainTime" = 0, "fineTuneTime" = 0)
   darch <- generateRBMs(darch,layers,genWeightFunc)
   return(darch)
-}
\ No newline at end of file
+}
